/*!
 * @file fi_bmp_planar.h
 * @brief Definitions and declarations for Fastec BMP planar images.
 *
 * Copyright 2020 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FIVID_FI_BMP_PLANER_
#define FIVID_FI_BMP_PLANER_

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <memory>
#include <vector>

#include "fas_video.h"
#include "fi_image.h"
#include "fi_bmp.h"

namespace FI_BMP
{
//--------------------------------------------------------------------
// Constant, Typedefs and Macros
//--------------------------------------------------------------------

//! The BMP magic value for Y8 Planar frame image.
extern const char *BMP_MAGIC_Y8;

//! The BMP magic value for NV12 Planar frame image.
extern const char *BMP_MAGIC_NV;

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class FiBMPPlanar : public FI_BMP::FiBMP
{
public:
    FiBMPPlanar();
    virtual ~FiBMPPlanar() {}

    virtual int CreateHeader(uint32_t fourcc, const pm_metadata *meta) override;
    std::unique_ptr<FAS_VIDEO::HEADER_DATA> GetHeader() override;

    virtual const uint8_t *GetAttachedFrameData(uint32_t& attachedFrameSize,
                                                FAS_VIDEO::FRAME_PLANAR_DATA& planarDataAttached,
                                                FAS_VIDEO::FRAME_ORIENTATION frameOrientation = FAS_VIDEO::FRAME_ORIENTATION::TOP_DOWN) override;

    //! @name Public Property Accessor Functions.
    //!
    //! These are the public functions used to access the properties of this object.
    //! @{

    virtual FAS_VIDEO::IMAGE_FORMAT GetImageFormat() override;
    virtual uint32_t GetImageWidth() override;
    virtual uint32_t GetImageHeight() override;
    virtual uint32_t GetBitsPerPixel() override;
    virtual uint32_t GetRowAlign() override;
    virtual uint32_t GetFullFramesize() override;
    virtual uint32_t GetHeaderSize() override;

    //! @}

protected:
    virtual bool VerifyImageData(const uint8_t *imageDataToVerify, uint32_t imageDataSize) override;
    virtual bool GetAttachedFramePlanarData(FAS_VIDEO::FRAME_PLANAR_DATA& planarDataToSet) override;
    virtual bool CopyBMPHeaderData(const uint8_t *imageDataToVerify, uint32_t imageDataSize) override;

    virtual bool IsFourCCValid(uint32_t fourCC, const char *whoIsCalling);

    bool isBmpY8(const char *imageDataPtr);
    bool isBmpNV(const char *imageDataPtr);

    bool headerExists();
};

} // End of FI_BMP

#endif  /* FIVID_FI_BMP_PLANER_ */

