/*!
 * @file fi_note.h
 * @brief Definitions and declarations for the Note object
 *    used by the Fastec Thread library.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FI_NOTE_H_
#define FI_NOTE_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------
#include <iostream>
#include <memory>
#include <sstream>
#include <string>
#include <cstring>

namespace FAS_THREAD
{
//--------------------------------------------------------------------
// Constant Definitions
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class Note
{
public:
    Note();
    explicit Note(const Note& n) noexcept;
    explicit Note(const char *f, int n, const char *fmt, ...);

    virtual ~Note() {}

    Note& operator<<(const char *s);

    void vappend(const char *file, int line, const char *fmt, va_list ap);
    void vappend(const char *fmt, va_list ap);

    virtual void append(const char *format, ...);
    virtual void append(const char *file, int line, const char *fmt, ...);

    virtual const std::string what();

    const std::string toString() const
    {
        return msg->str();
    }

protected:
    std::string get_datetime();

protected:
    std::unique_ptr<std::stringstream> msg; //!< The Note's text.
};

} /* namespace FAS_THREAD */

#endif /* FI_NOTE_H_ */
