/*!
 * @file fi_sync_message.h
 * @brief Definitions and declarations for the SyncMessage object(s)
 *    used by the Fastec Thread library.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FI_SYNC_MESSAGE_H_
#define FI_SYNC_MESSAGE_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <mutex>

#include "fas_thread.h"
#include "fi_message.h"

namespace FAS_THREAD
{

class SyncMessage: public Message
{
public:
    //! @name Construct/Destroy
    //! @{

    SyncMessage();
    virtual ~SyncMessage() {}
    //! @}

    virtual void doSync(FAS_THREAD::MsgThread *t);
    virtual void forward();

private:
    //! @brief Bookkeeping: true between doSync() and the final forward() call.
    bool msgBusy;

    //! @name %Thread-safety
    //! @{

    //! @brief Mutex to serialize access to SyncMessage state.
    std::unique_ptr<std::mutex> sync_mutex;

    //! @brief Condition variable on which the originating thread blocks.
    std::unique_ptr<std::condition_variable> sync_status;
    //!@}
};

} /* namespace FAS_THREAD */

#endif /* FI_SYNC_MESSAGE_H_ */
