/*
 * port.h
 *
 *  Created on: Dec 29, 2015
 *      Author: eric
 */

#ifndef INCLUDE_FASTEC_INTERNAL_SYS_PORT_H_
#define INCLUDE_FASTEC_INTERNAL_SYS_PORT_H_

#if defined(_WIN32) && !defined(__MINGW32__)
#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#else
#  include <unistd.h>
#endif

#include <time.h>

#ifdef __cplusplus
extern "C"
{
#endif

#if ! defined(__APPLE__)  &&  !(defined(_WIN32) && !defined(__MINGW32__))
#define fi_clock_gettime(id, pSpec) clock_gettime(id, pSpec)
#endif // ! defined(__APPLE__)  &&  ! defined(_WIN32)


#ifdef __APPLE__
/**
 * @brief Stand-in for POSIX timer stuff, for systems (Mac OS,
 * for example) that are missing these.
 */
#include <stdint.h>
#include <mach/mach_time.h>
#include <sys/time.h>

#define CLOCK_REALTIME  1
#define CLOCK_MONOTONIC 2

typedef int clockid_t;
typedef void* timer_t;

extern int fi_clock_gettime(clockid_t clk_id, struct timespec *tp);
#endif // __APPLE__


#if defined(_WIN32) && !defined(__MINGW32__)

#include <fastec/internal/sys/test_support.h>

#define drand48()	((double)rand()/RAND_MAX)
#define sleep(t)	Sleep(t*1000);
#define usleep(t)   Sleep(t/1000);

#define CLOCK_REALTIME  1
#define CLOCK_MONOTONIC 2

typedef int clockid_t;
typedef void* timer_t;

#if (_MSC_VER < 1900)
// For VS2013, this isn't defined. For VS2015 it is.
struct timespec
{
    time_t tv_sec;                /* Seconds */
    long   tv_nsec;               /* Nanoseconds */
};
#endif


struct itimerspec
{
    struct timespec it_interval;  /* Timer interval */
    struct timespec it_value;     /* Initial expiration */
};

extern FI_TEST_API int	fsync(int fd);
extern FI_TEST_API int	fi_clock_gettime(clockid_t clk_id, struct timespec *tp);
extern FI_TEST_API int	nanosleep(const struct timespec *req, struct timespec *rem);

#endif	// defined(_WIN32) && !defined(__MINGW32__)

#ifdef __cplusplus
}
#endif

#endif /* INCLUDE_FASTEC_INTERNAL_SYS_PORT_H_ */
